require 'yaml'
require 'ibanify/version'
require 'ibanify/countries.rb'

module Ibanify
  class IBAN
    def initialize(number)
      @number = IBAN.normalize(number)
    end

    def self.valid?(number)
      new(number).validation_error.nil?
    end

    def validation_error
      return :invalid_country_code unless countries[country_code]
      return :invalid_length       unless countries[country_code]['length'] == @number.size
      return :invalid_bban         unless bban =~ countries[country_code]['bban']
      return :invalid_check_digits unless valid_check_digits?
    end

    def valid_check_digits?
      iban = @number[4..-1] + @number[0, 4]
      iban.gsub!(/./) do |c|
        c.to_i(36)
      end
 
      iban.to_i % 97 == 1
    end

    def number
      @number
    end

    def country_code
      @number[0..1]
    end

    def bban
      @number[4..-1]
    end

    def countries
      load_countries_from_string(File.read(File.dirname(__FILE__) + '/ibanify/countries.yml'))
    end

    def load_countries_from_string(string)
      hash = YAML.load(string)
      hash.each do |country_code, rules|
        rules['bban'] = Regexp.new('^' + rules['bban'] + '$')
      end

      Countries.new(hash)
    end

    def self.normalize(number)
      number.strip.gsub(/\s+/, '').upcase
    end
  end
end
